"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Core = require("kamboja-core");
var invocation_1 = require("./invocation");
var factory_1 = require("./factory");
var framework_1 = require("../framework");
var index_1 = require("../binder/index");
var RequestHandler = /** @class */ (function () {
    function RequestHandler(facade, invocation) {
        var _this = this;
        this.facade = facade;
        var middlewares = (facade.middlewares || []).slice().reverse();
        if (invocation instanceof invocation_1.ControllerInvocation) {
            var controller = factory_1.ControllerFactory.resolve(invocation.controllerInfo, facade.dependencyResolver);
            middlewares.push.apply(middlewares, factory_1.MiddlewareFactory.resolveArray(framework_1.MiddlewareDecorator
                .getMiddlewares(controller), facade.dependencyResolver));
            middlewares.push.apply(middlewares, factory_1.MiddlewareFactory.resolveArray(framework_1.MiddlewareDecorator
                .getMiddlewares(controller, invocation.controllerInfo.methodMetaData.name), facade.dependencyResolver));
        }
        invocation.facade = facade;
        invocation.middlewares = middlewares;
        this.tail = this.head = invocation;
        middlewares.forEach(function (x) {
            _this.tail = new invocation_1.MiddlewareInvocation(x, _this.tail, middlewares, facade, invocation.controllerInfo);
        });
    }
    RequestHandler.prototype.execute = function (context, response, err) {
        var _this = this;
        return new Promise(function (resolve) { return resolve(_this.prepare(context, err)); })
            .then(function (result) { return _this.tail.proceed(); })
            .then(function (result) {
            if (context.contextType == "Handshake" && result.engine != "General")
                throw new Error("ActionResult Error, only return value of type 'redirect' and 'emit' are allowed in real time action");
            return result.execute(context, response, _this.tail.controllerInfo);
        })
            .catch(function (e) {
            response.send(new Core.ActionResult(e.message, e.status || 500));
        });
    };
    RequestHandler.prototype.prepare = function (context, err) {
        if (err) {
            this.tail.parameters = [err];
        }
        else if (context.contextType == "HttpRequest" && this.head.controllerInfo && this.head.controllerInfo.methodMetaData.parameters.length > 0) {
            var binder = new index_1.ParameterBinder(this.head.controllerInfo, this.facade.pathResolver);
            this.tail.parameters = binder.getParameters(context);
        }
        else if (context.contextType == "Handshake") {
            this.tail.parameters = [context.getPacket()];
        }
        this.tail.context = context;
    };
    return RequestHandler;
}());
exports.RequestHandler = RequestHandler;
